
open Ast;;

(* TYPES *)
type typed_compil_unit = {
  typed_package : path option;
  typed_imports : import list;
  typed_class : typed_class;
  typed_interf_decls : interf_decl list;
}

and inter_class = { (* intermediate type *)
  ic_name : string;
  ic_abstract : bool;  (* abstract class ? *)
  ic_father : inter_class option;
  mutable ic_children : inter_class list;
  ic_class : class_decl;
  mutable ic_fields : typed_field list;
  ic_methods : (int * int * string * method_def) list; (* relative superclass, id, method name *)
  ic_cons : (int * constructor) list; 
}

and typed_class = {
  tc_name : string;
  tc_father : typed_class option;
  mutable tc_children : typed_class list;
  tc_class : typed_class_decl;
  tc_field_count : int;
  tc_fields : typed_field list;
  (* todo : array with default values *)
  tc_method_count : int;
  (* relative superclass, id, method name , method*)
  tc_methods : (int * int * string * typed_method_def) array;
  tc_cons : typed_constructor array;
}

and typed_class_decl = {
  typed_class_modifiers : modifier list;
  typed_class_name : string;
  typed_class_extend : class_type;
  typed_class_implements : class_type list;
  typed_class_fields : typed_field list;
  typed_class_methods : typed_method_def list;
  typed_class_initializers : typed_instr list;
  typed_class_constructors : typed_constructor list;
}

(* class members *)

and typed_field = {
  typed_field_super_id : int;
  typed_field_id : int; (* field id relative to current class *)
  typed_field_modifiers : modifier list;
  typed_field_type : java_type;
  typed_field_name : string;
  typed_field_initializer : typed_init_expr;
}

and typed_method_def = {
  typed_method_modifiers : modifier list;
  typed_method_return : java_type option;
  typed_method_name : string;
  typed_method_mangled_name : string;
  typed_method_parameters : (java_type * string) list;
  typed_method_throws : class_type list;
  typed_method_body : typed_instr option;
}
and typed_methods = typed_method_def array
and typed_object = typed_class * typed_data array
and typed_data =
  | Void | Null 
  | Bool of bool | Int of int | Float of float 
  | String of string | Object of typed_object

  
and typed_constructor = {
  typed_constructor_modifiers : modifier list;
  typed_constructor_name : string;
  typed_constructor_parameters : (java_type * string) list;
  typed_constructor_throws : class_type list;
  typed_constructor_call : typed_ccall;
  typed_constructor_body : typed_instr option;
}

and typed_ccall = Cnone | Cthis of int * typed_exp list | Csuper of int * typed_exp list

and typed_init_expr = Simple_init of typed_exp | Array_init of typed_init_expr list

and typed_affect =
  | Alocal of java_type * int * int (* relative block, local variable id in block *)
  | Afield of java_type * int * int (* relative super class, variable id in class *)
  | Aparam of java_type * int  (* parameter index *)
  | Acplxname of string list
  | Acplxfield of expression * string
  | Acplxarray of expression * expression

and typed_method = 
  | Mmethod of method_def * int * int (* relative class, method_id *)
  
and typed_switch_label = Scase of typed_exp | Sdefault
and typed_instr =
  | Ilabeled of string * typed_instr
  | Iexpr of typed_exp
  | Idecl of java_type * string * typed_init_expr
  | Iblock of typed_instr list * typed_instr list (* declarations, instructions *)
  | Iifthenelse of typed_exp * typed_instr * typed_instr
  | Iswitch of typed_exp * (typed_switch_label list * (typed_instr list)) list
  | Iwhile of typed_exp * typed_instr
  | Idowhile of typed_instr * typed_exp
  | Ibreak of string option
  | Icontinue of string option
  | Ireturn of typed_exp option
  | Ithrow of typed_exp
  | Isynchronized of typed_exp * typed_instr
  | Itry of typed_instr * (java_type * string * typed_instr) list * typed_instr option
  | Inop
  | Iprintln of typed_exp

and typed_exp = java_type * exp_typed
and exp_typed = Eunary of Ast.unop * typed_exp
  | Ebinary of Ast.binop * typed_exp * typed_exp
  | Ebincomp of Ast.bincomp * typed_exp * typed_exp
  | Eternary of Ast.terop * typed_exp * typed_exp * typed_exp
  | Ecast of cast * typed_exp
  | Ebool of bool
  | Eint of int
  | Efloat of float
  | Estring of string
  | Enew_object of class_type * int * typed_exp list
  | Enew_array of java_type * expression list * int
  | Efield_access of typed_exp * string * int (* typed_exp * string * int *)
  | Ecall of string * int * typed_exp list
  | Eexpr_call of typed_exp * string * int * typed_exp list (* return method id *)
  | Earray_access of expression * expression
  | Etype_cast of java_type * int * expression
  | Eexpr_cast of expression * expression
  | Einstanceof of typed_exp * java_type
  | Eaffect of typed_affect * typed_exp
  | Evar of typed_affect
  | Enull
  | Ethis
  | Esuper

and cast = Float_of_int
  | Int_of_float | Int_of_int
  | String_of_int | String_of_float | String_of_bool | String_of_object
;;


(* ERROR MANAGEMENT *)
type typing_error =
    Clash of java_type * java_type
  | UndeclaredVariable of string
  | UndeclaredClass of string
  | UndeclaredMethod of string
  | RedeclaredVariableInBlock of string
  | RedeclaredVariableInMethod of string
  | RedeclaredVariableInClass of string
  | UnmanagedExpression of Ast.expression
  | UnexpectedType of java_type
  | InvalidImplicitCast of java_type * java_type
  | InvalidReturnType of java_type * java_type
  | NotAnObject of java_type
  | IncompatibleTypes of java_type * (java_type list)
  | CallingNonStaticFromStatic of string
  | CannotInstantiateAbstractClass of string
  | AbstractMethodInClass of string
  | AmbiguousCall of string * string * string list;;

exception Type_error of typing_error;;

